part of cresce_ui;

class CInputDropdown<T> extends StatefulWidget {
  final String label;
  final void Function(T value)? onChanged;
  final List<T> items;
  final bool setInitialValue;

  const CInputDropdown({
    Key? key,
    required this.label,
    required this.items,
    this.setInitialValue = false,
    this.onChanged,
  }) : super(key: key);

  @override
  State<CInputDropdown> createState() => _CInputDropdownState<T>();
}

class _CInputDropdownState<T> extends State<CInputDropdown> {
  late TextEditingController controller;

  @override
  void initState() {
    super.initState();
    controller = TextEditingController();
    controller.text =
        widget.items.isNotEmpty ? widget.items.first.toString() : '';
    if (widget.setInitialValue && widget.items.isNotEmpty) {
      (widget as CInputDropdown<T>).onChanged?.call(widget.items.first);
    }
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return CInputText(
      label: widget.label,
      field: controller,
      readonly: true,
      onTap: () async {
        var value = await _getSelectedItem(context);
        if (value == null) return;
        controller.text = value.toString();
        widget.onChanged?.call(value);
      },
    );
  }

  Future<T?> _getSelectedItem(BuildContext context) async {
    return await showDialog<T>(
      context: context,
      builder: (context) => _DropdownModal<T>(items: widget.items as List<T>),
    );
  }
}

class _DropdownModal<T> extends StatelessWidget {
  final List<T> items;

  const _DropdownModal({Key? key, required this.items}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Center(
      child: CSmallModal(
        size: Size(300, items.length * 50),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: items
              .map(
                (item) => GestureDetector(
                  onTap: () => Navigator.pop(context, item),
                  child: Padding(
                    padding: const EdgeInsets.symmetric(
                      vertical: 13,
                      horizontal: 20,
                    ),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(left: 8),
                          child: CText(
                            label: item.toString(),
                            style: CTextStyles.captionSecondaryVar1,
                          ),
                        ),
                        const CDivider(),
                      ],
                    ),
                  ),
                ),
              )
              .toList(),
        ),
      ),
    );
  }
}
